import gtk
class HypertextHelp(gtk.Window):
    hovering_over_link = False
    hand_cursor = gtk.gdk.Cursor(gtk.gdk.HAND2)
    regular_cursor = gtk.gdk.Cursor(gtk.gdk.XTERM)

    def __init__(self, parent=None):
        gtk.Window.__init__(self)
        self.set_screen(parent.get_screen())
        
        self.set_title('Help')
        self.set_default_size(450, 450)
        self.set_border_width(0)

        view = gtk.TextView()
        view.set_wrap_mode(gtk.WRAP_WORD)
        view.connect("key-press-event", self.key_press_event)
        view.connect("event-after", self.event_after)
        view.connect("motion-notify-event", self.motion_notify_event)
        view.connect("visibility-notify-event", self.visibility_notify_event)

        buffer = view.get_buffer()

        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.add(sw)
        sw.add(view)

        self.show_page(buffer, 1)

        self.show_all()

    # Links can be activated by pressing Enter.
    def key_press_event(self, text_view, event):
        if (event.keyval == gtk.keysyms.Return or
            event.keyval == gtk.keysyms.KP_Enter):
            buffer = text_view.get_buffer()
            iter = buffer.get_iter_at_mark(buffer.get_insert())
            self.follow_if_link(text_view, iter)
        return False

    # Links can also be activated by clicking.
    def event_after(self, text_view, event):
        if event.type != gtk.gdk.BUTTON_RELEASE:
            return False
        if event.button != 1:
            return False
        buffer = text_view.get_buffer()

        # we shouldn't follow a link if the user has selected something
        try:
            start, end = buffer.get_selection_bounds()
        except ValueError:
            # If there is nothing selected, None is return
            pass
        else:
            if start.get_offset() != end.get_offset():
                return False

        x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET,
            int(event.x), int(event.y))
        iter = text_view.get_iter_at_location(x, y)

        self.follow_if_link(text_view, iter)
        return False


    # Looks at all tags covering the position (x, y) in the text view,
    # and if one of them is a link, change the cursor to the "hands" cursor
    # typically used by web browsers.
    def set_cursor_if_appropriate(self, text_view, x, y):
        hovering = False

        buffer = text_view.get_buffer()
        iter = text_view.get_iter_at_location(x, y)

        tags = iter.get_tags()
        for tag in tags:
            page = tag.get_data("page")
            if page != 0:
                hovering = True
                break

        if hovering != self.hovering_over_link:
            self.hovering_over_link = hovering

        if self.hovering_over_link:
            text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(self.hand_cursor)
        else:
            text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(self.regular_cursor)

    # Update the cursor image if the pointer moved.
    def motion_notify_event(self, text_view, event):
        x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET,
            int(event.x), int(event.y))
        self.set_cursor_if_appropriate(text_view, x, y)
        text_view.window.get_pointer()
        return False

    # Also update the cursor image if the window becomes visible
    # (e.g. when a window covering it got iconified).
    def visibility_notify_event(self, text_view, event):
        wx, wy, mod = text_view.window.get_pointer()
        bx, by = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET, wx, wy)

        self.set_cursor_if_appropriate (text_view, bx, by)
        return False

    def insert_link(self, buffer, iter, text, page):
        ''' Inserts a piece of text into the buffer, giving it the usual
            appearance of a hyperlink in a web browser: blue and underlined.
            Additionally, attaches some data on the tag, to make it recognizable
            as a link.
        '''
        tag = buffer.create_tag(None,foreground="blue")
        tag.set_data("page", page)
        buffer.insert_with_tags(iter, text, tag)


    def show_page(self, buffer, page):
        ''' Fills the buffer with text and interspersed links. In any real
            hypertext app, this method would parse a file to identify the links.
        '''
        buffer.set_text("", 0)
        iter = buffer.get_iter_at_offset(0)
        if page == 1:
            buffer.insert(iter, "Help file explaining the ")
            self.insert_link(buffer, iter, "CHAMP", 2)
            buffer.insert(iter, " OBSERVATION PANEL\n\n")
            buffer.insert(iter, "svn checkout svn://bahamut.astro.lsa.unich.edu/CHAMP/champGui\n\n")
            buffer.insert(iter, "champGui is written in python and uses pygtk, numpy and matplotlib.\n\n")
            self.insert_link(buffer, iter, "Set server config\n", 11)
            self.insert_link(buffer, iter, "Set BC config\n", 7)
            self.insert_link(buffer, iter, "Set PZT parameters\n", 8)
            self.insert_link(buffer, iter, "Set FT parameters\n", 3)
            self.insert_link(buffer, iter, "Set/get offsets\n", 9)
            self.insert_link(buffer, iter, "Fringe Search\n", 4)
            self.insert_link(buffer, iter, "Fringe tracking\n", 5)
            self.insert_link(buffer, iter, "Plots\n", 6)
            self.insert_link(buffer, iter, "Champ RT commands\n", 10)
        elif page == 2:
            buffer.insert(iter,
                "CHARA Michigan Phase-tracker\n", -1)
            buffer.insert(iter,
                "Near IR fringe tracker. It modulates around fringe center by 1-2lambda "
                "at up to 500 Hz and calculate phase offsets in real-time using "
                "a modified ABCD method. Six pair-wise Mach-Zehnder combiners phase the entire Array\n\n", -1)
            self.insert_link(buffer, iter, "Go back", 1)
        elif page == 3:
            buffer.insert(iter,
                "FT parameters include gains, search SNR thresholds, semi-lock SNR thresholds "
                "and lock SNR thresholds. Can be set from Set FT parameters pop-up window (^P)\n\n"
                "Update server uses the CP Champ servo Parameters command\n\n", -1)
            self.insert_link(buffer, iter, "Go back", 1)
        elif page == 4:
            buffer.insert(iter,
                "Will follow a description of the fringe searching mode\n\n", -1)
            self.insert_link(buffer, iter, "Go back", 1)
        elif page == 5:
            buffer.insert(iter,
                "Will follow a description of the fringe tracking mode\n\n", -1)
            self.insert_link(buffer, iter, "Go back", 1)
        elif page == 6:
            buffer.insert(iter,
                "Will follow a description of the plots displayed in the GUI\n\n", -1)
            self.insert_link(buffer, iter, "Go back", 1)
        elif page == 7:
            buffer.insert(iter,
                "Utility to set the beam/telescope pairing.\n\n", -1)
            self.insert_link(buffer, iter, "Go back", 1)
        elif page == 8:
            buffer.insert(iter,
                "Utility to set the fringe tracker PZT parameters\n\n", -1)
            self.insert_link(buffer, iter, "Go back", 1)
        elif page == 9:
            buffer.insert(iter,
                "Utility to set/get offsets.\n"
                "SET BC REF OFF: sets the BC reference offsets for the selected beams based on the values provided.\n"
                "GET BC REF OFF: gets all the BC reference offsets from the current values held in the RT system.\n"
                "SET DELAY OFF: sets the delay offsets for the selected beams based on the values provided.\n\n", -1)
            self.insert_link(buffer, iter, "Go back", 1)
        elif page == 10:
            buffer.insert(iter,
                "CS: Champ Status\n"
                "Gets BC reference offsets, BC FT offsets and Champ status\n"
                "6 doubles, 6 doubles, 6 integers \n\n"
                "SS: Set Search parameters\n"
                "Sets BC search mode, speed and initial step (used by spiral search)\n"
                "6 integers, 6 floats, 6 doubles \n\n"
                "CP: Champ servo Parameters\n"
                "Sends gains, search SNR thresholds, semi-lock SNR thresholds and lock SNR thresholds\n"
                "6 floats, 6 floats, 6 floats, 6 floats\n\n"
                "AT: Set Amplitude thresholds\n"
                "6 floats \n\n"
                "CT: Calculate amplitude Thresholds\n"
                "Gets amplitude thresholds as calculated by RT champ\n"
                "6 floats \n\n"
                "FT: set Fringe Tracker piezzo parameters\n"
                "Sets dither frequency , dither amp, opd amp, dither smoothing frac and dither delay\n"
                "5 floats\n\n"
                "IR: Init Reference offsets\n"
                "Gets the current BC reference offsets from RT Champ\n"
                "6 doubles\n\n"
                "SR: Set Reference offsets\n"
                "Sends reference offsets to RT Champ and active(1)/inactive(999) flag\n"
                "6 integers, 6 doubles\n\n"
                "DO: Set Delay Offsets\n"
                "Sends delay offsets to RT Champ and active(1)/inactive(999) flag\n"
                "6 integers, 6 doubles\n\n"
                "PA: PAddle the fringes\n"
                "Sends paddle fringe offsets to RT Champ\n"
                "1 integer, 1 double\n\n"
                "RC: get Champ detector Config\n"
                "\n\n",
                -1)
            self.insert_link(buffer, iter, "Go back", 1)
        elif page == 11:
            buffer.insert(iter,
                "Set Champ RT server parameters: host name and port number. As well as the socket timeout.\n\n", -1)
            self.insert_link(buffer, iter, "Go back", 1)


    def follow_if_link(self, text_view, iter):
        ''' Looks at all tags covering the position of iter in the text view,
            and if one of them is a link, follow it by showing the page identified
            by the data attached to it.
        '''
        tags = iter.get_tags()
        for tag in tags:
            page = tag.get_data("page")
            if page != 0:
                self.show_page(text_view.get_buffer(), page)
                break
            
